package hexIT;

import java.lang.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;

/**
 *   This class represents a configuration of hexagons, a "board". There can
 *   be many possibilities for how to layout the Hexagons(tiles) on the board.
 *   These specific arrangements are abstractly defined here, and left to be
 *   implemented by subclasses. This base class contains all the functions that
 *   can be used by all board types.
 *
 * @author Marc Lanctot
 * @version 1.0
 * @last modified: Oct 6, 2003
 * @changes: comments added
 */
public abstract class Board implements java.io.Serializable
{
   // tiles/orientation size
   protected int size;

   protected JFrame  frame;
   protected int frameWidth, frameHeight;

	/** Creates a new, empty Board. */
   public Board()
   {
   }

	/**
	 *   Creates a new board of the specified size. The semantics of this size
	 *   setting depends on the board type (sub type).
	 */
   public Board(int new_size)
   {
      size= new_size;
   }

	/**
	 *   Creates a Frame to view this board in. By default the frame is not
	 *   visible.
	 */
   public void createFrame(String name)
   {
      frame= new JFrame(name);
      frame.setVisible(false);
   }

	/**
	 *   Sets the size of the frame, in pixels, to the specified values.
	 */
   public void setFrameSize(int new_width, int new_height)
   {
      frameWidth= new_width;
      frameHeight= new_height;
      frame.setSize(frameWidth, frameHeight);
   }

	/**
	 *   Show the internal frame created by createFrame().
	 */
   public void show()
   {
      // scale hexagons & images

      frame.setVisible(true);
   }

	/**
	 *   Gets all the hexes surrounding the hexes at hex coords (x,y) within a
	 *   specified radius (in hex units).
	 */
   public abstract java.util.List getSurrounding(int hexx, int hexy, int radius);


	/**
	 *   Gets all the hexes that can be "seen" from one hexagon to another. "Seen"
	 *   means all the hexes that intersect a straight line drawn from the source
	 *   hexagon to the destination hexagon. (hexx1,hexy1) are the hex coordinates
	 *   for the source while (hexx2, hexy2) are the hex coordinates for the destination.
	 */
   public abstract java.util.List getLOS(int hexx1, int hexy1, int hexx2, int hexy2);

	/**
	 *   Gets all the hexes that can be "seen" in the Field-of-View of an observer at
	 *   hex coordinate (hexx1,hexy1). The FOV is all the hexagons that can be seen from
	 *   a 360 degree cone spawning out from the center of the source hexagon. Note that
	 *   objects are considered to take up the full hexagons they are on, and block whatever
	 *   us behind them with respect to the observer. The maxradius is the "range" of sight
	 *   or view.
	 */
   public abstract java.util.List getFOV(int hexx1, int hexy1, int maxradius);

	/**
	 *   Returns the shortest path (a list of haxagons) from hex locations (hexy1,hexy2)
	 *   to (hexx2, hexy2);
	 */
   public abstract java.util.List getShortestPath(int hexx1, int hexy1, int hexx2, int hexy2);

	/**
	 *   Returns the exterior of the hexagonal board. What exactly the exterior is depends
	 *   on the board type, but should be the outer layer of hexagons: the ones that form
	 *   the end of the board.
	 */
	public abstract java.util.List getExterior();

   // ---------------------------------------------------------------------------------------------

   /**
	 * A distance metric between 2 separate hexagons. This is the minimum # of hex tiles
    * it takes to walk from one hex to the other. It is also the length of the shortest
	 * path.
	 */
   public abstract int distance(Hexagon h1, Hexagon h2);

   /**
	 *  Returns an iterator over all hexagons in the board, in no particular order.
	 */
   public abstract Iterator tilesIterator();

   /**
	 *  Finds an object that is on one of the hexagons. Returns the hexagon on which the
	 *  object is located, or null if not found. Note: this algortihm compares
	 *  via reference values, <b>not</b> by using equals().
	 */
   public Hexagon findObject(Drawable obj)
   {
      Iterator i= tilesIterator();

      while (i.hasNext())
      {
         Hexagon h= (Hexagon)i.next();

         if (h.getObj() == obj)
         {
            return h;
         }
      }

      return null;
   }





}
