package hexIT;

import java.lang.*;
import java.awt.*;

/**
 *  A hexagon object represents a particular tile on a board of hexagons.
 *  The board type is independent of the implementation found here: hexagons
 *  have 6 sides no matter what arrangement they are placed in.
 *
 * @author Marc Lanctot
 * @version 1.0
 * @last modified: Oct 6, 2003
 * @changes: comments added
 */
public class Hexagon implements java.io.Serializable, Cloneable
{
	public static final int COORDNUM_TOP=              0;
	public static final int COORDNUM_TOPRIGHT=         1;
	public static final int COORDNUM_BOTTOMRIGHT=      2;
	public static final int COORDNUM_BOTTOM=           3;
	public static final int COORDNUM_BOTTOMLEFT=       4;
	public static final int COORDNUM_TOPLEFT=          5;

   // there can be a drawable object associated with this tile. the semantics obviously
   // depends on the application.. this is setup for a most general case. eg, for a game of 'hexwars',
   // this object can be a player's stone.
   private Drawable object;

   // pos of top-left corner (pixels)
   private int curX, curY;

   // pos inside representation of the board
   private int hexX, hexY;

   private boolean visible;
   private Color borderColor;
   private Color outlineColor;
   private Color fillColor;

   // 'coordnums'(indices of these arrays) are ids for vertices of a hexagons: from topmost point
   // going clock clockwise: 0-5
   private int[] pixelX, pixelY;

   // ---------------------------------------------------------------------------------------------

	/**
    *  Creates a hexagon with hexagonal coordinates (new_hexX, new_hexY). Note that
	 *  these coordinates are in the coordinate system defined by the board. Constructing
	 *  a hexagon object does not place it on any board. These coordinates are merely
	 *  convenient records store in the hexagon object.
	 */
   public Hexagon(int new_hexX, int new_hexY)
   {
      // by default, no images on the hexagon
      hexX= new_hexX;
      hexY= new_hexY;

      borderColor= Color.gray;
      outlineColor= null;
      fillColor= null;
      

      visible= false;
      
      pixelX= new int[6];
      for (int i= 0; i < 6; i++)
         pixelX[i]= -1;
      
      pixelY= new int[6];
      for (int i= 0; i < 6; i++)
         pixelY[i]= -1;
   }

	/**
	 *  Get this hexagon's hexagonal X coordinate, where the coordinate system
	 *  is defined by the board that created this hexagon.
	 */
   public int getHexX()
   {
      return hexX;
   }

	/**
	 *  Get this hexagon's hexagonal Y coordinate, where the coordinate system
	 *  is defined by the board that created this hexagon.
	 */
   public int getHexY()
   {
      return hexY;
   }

	/**
	 *  Get the x coordinate of the pixel attached to the coordinate number
	 *  of the vertex of this hexagon.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public int getPixelX(int coord_num)
   {
      return pixelX[coord_num];
   }

	/**
	 *  Get the y coordinate of the pixel attached to the coordinate number
	 *  of the vertex of this hexagon.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public int getPixelY(int coord_num)
   {
      return pixelY[coord_num];
   }

   protected void setPixelX(int coord_num, int val)
   {
      pixelX[coord_num]= val;
   }

   protected void setPixelY(int coord_num, int val)
   {
      pixelY[coord_num]= val;
   }

   /**
	 *  Get the x coordinate of the pixel at the center of this hexagon.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public int getMidPixelX()
   {
      return pixelX[0];
   }

   /**
	 *  Get the y coordinate of the pixel at the center of this hexagon.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public int getMidPixelY()
   {
      return (pixelY[1] + pixelY[2])/2;
   }

	/**
	 *  Places the specified drawable object onto this hex. Only 1 object is
	 *  currently allowed on a hexagon. If you call this function while another
	 *  object is currently on this hexagon, the old object will be replaced.
	 */
   public void putObj(Drawable newObj)
   {
      object= newObj;
   }

	/** Return the object on this hexagon, or null if there is none. */
   public Drawable getObj()
   {
      return object;
   }

	/** Returns true if there is an object on this hexagon, false otherwise. */
	public boolean isOccupied()
	{
		return (getObj() != null);
	}

	/**
	 *  Scales the object on this hexagon to the specified new pixel values.
	 */
   public void scaleObjImage(int new_width, int new_height)
   {
      getObj().setImage(ImageTools.scaleImage(getObj().getImage(), new_width, new_height));
   }

	/**
	 *  Remove the object on this hexagon. If there is none, this method does
	 *  nothing.
	 */
   public void removeObj()
   {
      object= null;
   }

	/**
	 *  Set the border line color of this hexagon.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public void setBorderColor(Color new_color)
   {
      borderColor= new_color;
   }

	/**
	 *  Returns the current border color of this hexagon.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public Color getBorderColor()
   {
      return borderColor;
   }

	/**
	 *  Set the outline line color of this hexagon. The outline color
	 *  is a line that surrounds the interior edge of the hexagon, so
	 *  it is not coincident with the border.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   protected void setOutlineColor(Color new_color)
   {
      outlineColor= new_color;
   }

	/**
	 *  Gets the outline line color of this hexagon. The outline color
	 *  is a line that surrounds the interior edge of the hexagon, so
	 *  it is not coincident with the border.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public Color getOutlineColor()
   {
      return outlineColor;
   }

	/**
	 *  Returns true if this hexagon has an outline, false otherwise.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public boolean hasOutline()
   {
      return (outlineColor != null);
   }

	/**
	 *  Remove the outline from this hexagon. If the hexagon does not
	 *  currently have an outline, this method does nothing.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public void removeOutline()
   {
      outlineColor= null;
   }

	/**
	 *  Returns true if the specified pixel with coordinates (x,y)
	 *  lie inside this hexagon.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public boolean surroundsPixel(int x, int y)
   {
      float slope= ((float)pixelY[5]-(float)pixelY[0])
		              /((float)pixelX[0]-(float)pixelX[5]);
      
      // left half
      if (x >= pixelX[5] && x <= pixelX[0])
      {
         int y_offset= (int)(slope*((float)(x-pixelX[5])));

         if (y >= (pixelY[5] - y_offset) && y <= (pixelY[4] + y_offset))
         {
            return true;
         }
      }
      else if (x > pixelX[0] && x <= pixelX[1])
      {
         int y_offset= (int)(slope*((float)(pixelX[1]-x)));

         if (y >= (pixelY[1] - y_offset) && y <= (pixelY[2] + y_offset))
         {
            return true;
         }
      }
      
      return false;      
   }

   protected void printCoords()
   {
      System.out.println("("+pixelX[0]+","+pixelY[0]+") ("+pixelX[1]+","+pixelY[1]+") ("+pixelX[2]+","+pixelY[2]+") ("
         +pixelX[3]+","+pixelY[3]+") ("+pixelX[4]+","+pixelY[4]+") ("+pixelX[5]+","+pixelY[5]+")");
   }

	/**
	 *  Sets the fill color of this hexagon. If this hex also has an outline,
	 *  the outline will be also be displayed.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   protected void setFillColor(Color new_color)
   {
      fillColor= new_color;
   }

	/**
	 *  Gets the fill color of this hexagon.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public Color getFillColor()
   {
      return fillColor;
   }

	/**
	 *  Returns true if this hexagon has a fill color, false otherwise.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public boolean hasFill()
   {
      return (fillColor != null);
   }

	/**
	 *  Removes the fill from this hexagon.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public void removeFill()
   {
      fillColor= null;
   }

	/**
	 *  Toggles the visibility of this hexagon.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public void setVisible(boolean new_visible)
   {
      visible= new_visible;
   }

	/**
	 *  Returns true if this hexagon is visible, false otherwise.
	 *  <br><br>
	 *  <b>Note</b>: this function will only does something
	 *  meaningful if the hexagon in question is part of a board which
	 *  is a part of a hex panel.
	 */
   public boolean isVisible()
   {
      return visible;
   }

	/**
	 *  Resets the hexagonal coordinates of this hexagon. Remember,
	 *  these coordinates only have meaning when associated with a
	 *  board's coordinate system.
	 */
   protected void place(int new_X, int new_Y)
   {
      curX= new_X;
      curY= new_Y;
   }

	/**
	 *  Returns true if the object that's passed-in is a Hexagon and
	 *  has the same hexagonal coordinates as this Hexagon.
	 */
   public boolean equals(Object o)
   {
      if (!(o instanceof Hexagon))
         return false;
      
      Hexagon h2= (Hexagon)o;
      
      return (hexX == h2.hexX && hexY == h2.hexY);
   }

	/** Returns the coordinates string of this hexagon. */
	public String toString()
	{
		return "(" + getHexX() + "," + getHexY() + ")";
	}


}

