package hexIT;

import java.lang.*;
import java.util.*;     // for Iterator

/**
 *  A hexagonal board is a board which is itself arranged into one
 *  big hexagon composed of smaller ones. A hexagonal board of size
 *  3 is depicted below.
 *
 * <pre>
 * Coord System & Example:
 *                                          y-coord
 *                                 0 1 2
 *                                / / / 3
 *                            0- . . . / 4
 *                           1- . . . . /
 *             x-coord      2- . . . . .
 *                           3- . . . x    <-- that 'x' is at position (3,4)
 *                            4- . . .
 *
 *
 * </pre>
 *
 * @author Marc Lanctot
 * @version 1.0
 * @last modified: Oct 6, 2003
 * @changes: comments added
 */
public class HexagonalBoard extends Board implements java.io.Serializable, Cloneable
{
 // notes:
 //   - we need a (2*size-1) by (2*size-1) array to store all hexagons.
 //     the representation of this hexagonal board quite naturally fits into a 2D array
 //     - some of the array is wasted, but this it's worth the ease-of-use

   protected Hexagon[][]   tiles;     // coords stored as [x][y]
   protected int sideLength;          // in tiles

   public static final int DIR_NW= 0,
                           DIR_NE= 1,
                           DIR_W= 2,
                           DIR_E= 3,
                           DIR_SW= 4,
                           DIR_SE= 5;

	/** Create an empty hexagonal board with no size. */
   public HexagonalBoard()
   {
   }

	/** Create a hexagonal board of the specified size. */
   public HexagonalBoard(int new_size)
   {
      super(new_size);

      sideLength= 2*size-1;
      tiles= new Hexagon[sideLength][sideLength];

      // create the empty tiles
      for (int i= 0; i < sideLength; i++)
         for (int j= 0; j < sideLength; j++)
            tiles[i][j]= new Hexagon(i, j);
   }

   /**
	 *  Returns the minimum # of hexes need to travel from one hexagon to the other.
	 */
   public int distance(Hexagon h1, Hexagon h2)
   {
      // this method may only work for dist <= 3 (haven't tested higher than that)
      int deltaX= h2.getHexX()-h1.getHexX();
      int deltaY= h2.getHexY()-h1.getHexY();

      if (   (Misc.sign(deltaX) == -1 && Misc.sign(deltaY) == 1)
          || (Misc.sign(deltaX) == 1 && Misc.sign(deltaY) == -1) )
         return (Misc.abs(deltaX) + Misc.abs(deltaY));

      return Misc.max(Misc.abs(deltaX),Misc.abs(deltaY));
   }

	/**
	 *  Returns true if the specified hexagonal coordinates are not valid.
	 *  An example of an invalid coordinate is on that does not resolve to
	 *  a position on an actual board. Using the above example of a hexagonal
	 *  board of size 3, (-1, -1) is not valid, (3,0) is not valid, (100,1)
	 *  is not valid, but (1,1) is valid.
	 */
   public boolean isValidCoord(int x, int y)
   {
      int median= (sideLength/2);

      if (x < 0 || x >= sideLength || y < 0 || y >= sideLength)
         return false;
      else if (x > median && x > y && (x-y) > median)         // bottom left region
         return false;      
      else if (y > median && y > x && (y-x) > median)         // top right region
         return false;
      
      return true;
   }

	/**
	 *  Return the hexagon at hex location (x,y) or null if invalid
	 *  coordinates, or null if the location is not valid.
	 */
   public Hexagon getHexagon(int x, int y)
   {
		if (isValidCoord(x,y))
      	return tiles[x][y];

		return null;
   }

	/**
	 *  Returns the side length. The side length is the number of valid
	 *  hexagons on one axis: 2*size-1 in the case of these types of
	 *  boards.
	 */
   public int getSideLength()
   {
      return sideLength;
   }

	/** Move the object at hex location h1 to h2. */
	public void moveObject(Hexagon h1, Hexagon h2)
	{
		moveObject(h1.getHexX(), h1.getHexY(), h2.getHexX(), h2.getHexY());
	}
	
	/** Move the object at hex location (x1,y1) to (x2,y2). */
   public void moveObject(int x1, int y1, int x2, int y2)
   {
      if (!isValidCoord(x1, y1))
      {
         System.out.println("Invalid coordinates: (x1, y1) = (" + x1 + "," + y1 + ") in HexagonalBoard.moveObject()");
         return;
      }
      else if (!isValidCoord(x2, y2))
      {
         System.out.println("Invalid coordinates: (x2, y2) = (" + x2 + "," + y2 + ") in HexagonalBoard.moveObject()");
         return;
      }         
      else if (getHexagon(x1, y1).getObj() == null)
      {
         System.out.println("No object on source hexagon in HexagonalBoard.moveObject()");
         return;
      }
      else if (getHexagon(x2, y2).getObj() != null)
      {
         System.out.println("Object(collision) on destination hexagon in HexagonalBoard.moveObject()");
         return;
      }
         
      Drawable obj= getHexagon(x1, y1).getObj();
      getHexagon(x2, y2).putObj(obj);
      getHexagon(x1, y1).removeObj();      
   }

	/** Move the object at hex location (x,y) in the specified direction. */
   public void moveObject(int x, int y, int direction)
   {
      switch(direction)
      {
         case HexagonalBoard.DIR_NW:
         {
            moveObject(x, y, x-1, y-1);
            break;
         }
         case HexagonalBoard.DIR_NE:
         {
            moveObject(x, y, x-1, y);
            break;
         }
         case HexagonalBoard.DIR_W:
         {
            moveObject(x, y, x, y-1);
            break;
         }
         case HexagonalBoard.DIR_E:
         {
            moveObject(x, y, x, y+1);
            break;
         }
         case HexagonalBoard.DIR_SW:
         {
            moveObject(x, y, x+1, y);
            break;
         }
         case HexagonalBoard.DIR_SE:
         {
            moveObject(x, y, x+1, y+1);
            break;
         }
         default: break;
      }
   }

	/** Put the specified object on hex location (x,y). */
   public void putObject(int x, int y, Drawable obj)
   {
      getHexagon(x,y).putObj(obj);
   }

	/** Remove the object on hex location (x,y). */
   public void removeObject(int x, int y)
   {
      getHexagon(x,y).removeObj();
   }

	/**
	 *  Returns an iterator over all hexagons on the board.
	 */
   public Iterator tilesIterator()
   {
      return new TileIterator(this);
   }

	/**
	 *  Gets the hexagon which contains this pixel.
	 */
   public Hexagon getHexagonByPixel(int pixx, int pixy)
   {
		// this should be more efficient. see TODO.txt

      Iterator i= tilesIterator();

      while (i.hasNext())
      {
         Hexagon h= (Hexagon)i.next();
         if (h.surroundsPixel(pixx, pixy))
            return h;
      }

      return null;
   }
	
	/** 
	 *  Returns an array of all valid hexagons on this hexagonal board, 
	 *  in no particular order.
	 */
	public Hexagon[] getHexagons() 
	{
		List l= new LinkedList();

		Iterator i= tilesIterator();

		while (i.hasNext())
		{
			Hexagon h= (Hexagon)i.next();
			l.add(h);
		}

		Object[] array= l.toArray();
		Hexagon[] array2= new Hexagon[array.length];

		for (int j= 0; j < array.length; j++)
			array2[j]= (Hexagon)array[j];

		return array2;
	}

	/** 
	 * Returns an array of all valid hexagon on this hexagonal board that 
	 * are occupied by pieces, in no particular order.
	 */
	public Hexagon[] getOccupiedHexagons() 
	{
		List l= new LinkedList();

		Iterator i= tilesIterator();

		while (i.hasNext())
		{
			Hexagon h= (Hexagon)i.next();
			if (h.isOccupied())
				l.add(h);
		}

		Object[] array= l.toArray();
		Hexagon[] array2= new Hexagon[array.length];

		for (int j= 0; j < array.length; j++)
			array2[j]= (Hexagon)array[j];

		return array2;
	}

	/** 
	 * Returns an array of all valid hexagon on this hexagonal board that 
	 * are not occupied by pieces, in no particular order.
	 */
	public Hexagon[] getNonOccupiedHexagons() 
	{
		List l= new LinkedList();

		Iterator i= tilesIterator();

		while (i.hasNext())
		{
			Hexagon h= (Hexagon)i.next();
			if (!h.isOccupied())
				l.add(h);
		}

		Object[] array= l.toArray();
		Hexagon[] array2= new Hexagon[array.length];

		for (int j= 0; j < array.length; j++)
			array2[j]= (Hexagon)array[j];

		return array2;
	}

	/**
	 *  Returns the list of hexagons that forms the shortest path between
	 *  the source hexagon at (hexx1,hexy1) and destination hexagon at
	 *  (hexx2,hexy2).
	 */
   public List getShortestPath(int hexx1, int hexy1, int hexx2, int hexy2)
	{
      // this method may only work for dist <= 3 (haven't tested higher than that)
      int deltaX= hexx2-hexx1;
      int deltaY= hexy2-hexy1;
		int stepX, stepY, dist, distX, distY, dir, x, y;

		List hexList= new LinkedList();

      if (Misc.sign(deltaX) == -1 && Misc.sign(deltaY) == 1)
		{
			hexList.add(getHexagon(hexx1, hexy1));

			for (stepX= -1; (hexx1 + stepX) >= hexx2; stepX--)
				hexList.add(getHexagon(hexx1 + stepX, hexy1));

			stepX++;

			for (stepY= +1; (hexy1 + stepY) <= hexy2; stepY++)
				hexList.add(getHexagon(hexx1 + stepX, hexy1 + stepY));
      }
		else if (Misc.sign(deltaX) == 1 && Misc.sign(deltaY) == -1)
		{
			hexList.add(getHexagon(hexx1, hexy1));

			for (stepY= -1; (hexy1 + stepY) >= hexy2; stepY--)
				hexList.add(getHexagon(hexx1, hexy1 + stepY));

			stepY++;

			for (stepX= +1; (hexx1 + stepX) <= hexx2; stepX++)
				hexList.add(getHexagon(hexx1 + stepX, hexy1+stepY));
		}
		else if (   (Misc.sign(deltaX) == -1 || Misc.sign(deltaY) == -1)
		         || (Misc.sign(deltaX) == 1 || Misc.sign(deltaY) == 1)  )
		{
			boolean positive= true;

			if (Misc.sign(deltaX) == -1 || Misc.sign(deltaY) == -1)
				positive= false;

			dist= distance(getHexagon(hexx1, hexy1), getHexagon(hexx2, hexy2));
			distX= Misc.abs(deltaX);
			distY= Misc.abs(deltaY);
			x= -1;
			y= -1;

			if (distX >= distY)
			{
				for (stepX= 0;
				     (dir= isStraightPath(hexx1+stepX, hexy1, hexx2, hexy2)) == -1;
				     stepX= (positive? (stepX+1) : (stepX-1)))
					hexList.add(getHexagon(hexx1+stepX, hexy1));

				x= hexx1+stepX;
				y= hexy1;
			}
			else
			{
				for (stepY= 0;
				     (dir= isStraightPath(hexx1, hexy1+stepY, hexx2, hexy2)) == -1;
				     stepY= (positive? (stepY+1) : (stepY-1)))
					hexList.add(getHexagon(hexx1, hexy1+stepY));

				x= hexx1;
				y= hexy1+stepY;
			}

			while (!(x == hexx2 && y == hexy2))
			{
				hexList.add(getHexagon(x, y));
				x= nextX(x, dir);
				y= nextY(y, dir);
			}

			hexList.add(getHexagon(hexx2, hexy2));
		}
		else
		{
			hexList.add(getHexagon(hexx1, hexy1));
		}

		return hexList;
	}

	

	/** Returns the list of hexagons on the outer edges. */
	public List getExterior()
	{
		LinkedList ll= new LinkedList();

		int x= 0;
		int y= 0;

		for (x= 0; x < size; x++)
			ll.add(getHexagon(x,y));

		for (x= size, y= 1; x < (2*size-1); x++, y++)
			ll.add(getHexagon(x,y));

		for (x--; y < (2*size-1); y++)
			ll.add(getHexagon(x,y));

		for (y--, x= x-1; x >= size; x--)
			ll.add(getHexagon(x,y));

		for (; x >= 0; x--, y--)
			ll.add(getHexagon(x,y));

		for (x++; y >= 0; y--)
			ll.add(getHexagon(x,y));

		return ll;
	}

	/** 
	 * Returns the hexagon which is of distance 1 away in the specified direction
	 * from the specified Hexagon. If the coordinates are invalid, returns null.
	 */
	public Hexagon nextHexagon(Hexagon h, int dir)
	{
		int newx= nextX(h.getHexX(), dir);		
		int newy= nextY(h.getHexY(), dir);		

		if (!isValidCoord(newx, newy))
			return null;
		else
			return getHexagon(newx, newy);
	}
	
	/**
	 *  Returns the next value of the hex x-coordinate given
	 *  the current value x and moving in the specified direction.
	 */
	public int nextX(int x, int dir)
	{
		switch(dir)
		{
			case HexagonalBoard.DIR_NW: return (x-1);
			case HexagonalBoard.DIR_NE: return (x-1);
			case HexagonalBoard.DIR_W: return (x);
			case HexagonalBoard.DIR_E: return (x);
			case HexagonalBoard.DIR_SW: return (x+1);
			case HexagonalBoard.DIR_SE: return (x+1);
			default: return -10000;
		}
	}

	/**
	 *  Returns the next value of the hex y-coordinate given
	 *  the current value x and moving in the specified direction.
	 */
	public int nextY(int y, int dir)
	{
		switch(dir)
		{
			case HexagonalBoard.DIR_NW: return (y-1);
			case HexagonalBoard.DIR_NE: return (y);
			case HexagonalBoard.DIR_W: return (y-1);
			case HexagonalBoard.DIR_E: return (y+1);
			case HexagonalBoard.DIR_SW: return (y);
			case HexagonalBoard.DIR_SE: return (y+1);
			default: return -10000;
		}
	}

	/**
	 * If there is a straight line of hexagons (ie: you can get there by going
	 * in one constant direction) between the source and destination, then the
	 * direction (from source to dest) is returned. Otherwise, -1 is returned.
	 */
	public int isStraightPath(Hexagon h1, Hexagon h2)
	{
		return isStraightPath(h1.getHexX(), h1.getHexY(), h2.getHexX(), h2.getHexY());
	}
	
	/**
	 * If there is a straight line of hexagons (ie: you can get there by going
	 * in one constant direction) between the source and destination, then the
	 * direction (from source to dest) is returned. Otherwise, -1 is returned.
	 */
	public int isStraightPath(int hexx1, int hexy1, int hexx2, int hexy2)
	{
		int d_hexx= (hexx2 - hexx1);
		int d_hexy= (hexy2 - hexy1);

		if (d_hexx == 0 && d_hexy != 0)
		{
			if (d_hexy > 0)
				return HexagonalBoard.DIR_E;
			else if (d_hexy < 0)
				return HexagonalBoard.DIR_W;
		}
		else if (d_hexx != 0 && d_hexy == 0)
		{
			if (d_hexx > 0)
				return HexagonalBoard.DIR_SW;
			else if (d_hexx < 0)
				return HexagonalBoard.DIR_NE;
		}
		else if (d_hexx != 0 && d_hexy != 0 && d_hexx == d_hexy)
		{
			if (d_hexx < 0)
				return HexagonalBoard.DIR_NW;
			else if (d_hexx > 0)
				return HexagonalBoard.DIR_SE;
		}

		return -1;
	}

	/**
	 *  Returns the line-of-sight from hexagon at hex location (hexx1,hexy1).
	 *  This is done by drawing a line between the middle pixel of the source
	 *  hexagon to the pixel (iBx,iBy) and seeing which hexagons intersect with
	 *  the given line. A list of the intersecting hexagons is returned.
	 */
	public List getLOSPixel(int hexx1, int hexy1, int iBx, int iBy)
	{
		// the algo here was taken from
		// http://www.exaflop.org/docs/cgafaq/cga1.html
		//
		//      (Ay-Cy)(Dx-Cx)-(Ax-Cx)(Dy-Cy)
		//  r = -----------------------------  (eqn 1)
		//      (Bx-Ax)(Dy-Cy)-(By-Ay)(Dx-Cx)
		//
		//     (Ay-Cy)(Bx-Ax)-(Ax-Cx)(By-Ay)
		// s = -----------------------------  (eqn 2)
		//     (Bx-Ax)(Dy-Cy)-(By-Ay)(Dx-Cx)
		//
		// If 0<=r<=1 & 0<=s<=1, then intersect
		//
		// Let A = source pixel, B = dest pixel
		// C and D will be the endpoints of each side on our hexes
		//
		// in our A and B never change,
		//   so let ABx= Ax-Bx, BAx= Bx-Ax, ABy= Ay-By, BAy= By-Ay
		//
		// so then r=  ACy*DCx-ACx*DCy
		//             ----------------------------
		//             BAx*DCy-BAy*DCx
		//
		//         s=  ACy*BAx - ACx*BAy
		//             ------------------------
		//             BAx*DCy - BAy*DCx

		List hexList= new LinkedList();
		Hexagon source, curHex;
		source= getHexagon(hexx1, hexy1);
		curHex= source;
		hexList.add(source);

		double Ax= (double)source.getMidPixelX();
		double Ay= (double)source.getMidPixelY();
		double Bx= (double)iBx;
		double By= (double)iBy;
		double Cx, Cy, Dx, Dy, r, s, DCx, DCy, ACx, ACy, denom;

		boolean found= false;
		int coordnum;
		int c1, c2, entered= -1;

		// constants
		double ABx= Ax-Bx;
		double BAx= Bx-Ax;
		double ABy= Ay-By;
		double BAy= By-Ay;

		do
		{
			// check which side the line is exiting
			for (coordnum= 0; coordnum < 6; coordnum++)
			{
				if (coordnum == entered)
					continue;  // don't check where we came in!

				c1= coordnum;
				c2= coordnum+1; if (c2 == 6) c2= 0;

				Cx= curHex.getPixelX(c1);
				Cy= curHex.getPixelY(c1);
				Dx= curHex.getPixelX(c2);
				Dy= curHex.getPixelY(c2);

				DCx= Dx-Cx;
				DCy= Dy-Cy;
				ACx= Ax-Cx;
				ACy= Ay-Cy;

				denom= BAx*DCy - BAy*DCx;

				//System.out.println("denom = " + denom);

				r= (ACy*DCx - ACx*DCy) / denom;
				s= (ACy*BAx - ACx*BAy) / denom;

				//System.out.println("r = " + r + ", s = " + s);

				if (r >= 0.0 && r <= 1.0 && s >= 0.0 && s <= 1.0)
				{
					// intersection!
					int x= curHex.getHexX();
					int y= curHex.getHexY();

					switch (coordnum)
					{
						case Hexagon.COORDNUM_TOP:
							curHex= getHexagon(x-1, y);
							entered= Hexagon.COORDNUM_BOTTOM;
							break;

						case Hexagon.COORDNUM_TOPRIGHT:
							curHex= getHexagon(x, y+1);
							entered= Hexagon.COORDNUM_BOTTOMLEFT;
							break;

						case Hexagon.COORDNUM_BOTTOMRIGHT:
							curHex= getHexagon(x+1, y+1);
							entered= Hexagon.COORDNUM_TOPLEFT;
							break;

						case Hexagon.COORDNUM_BOTTOM:
							curHex= getHexagon(x+1, y);
							entered= Hexagon.COORDNUM_TOP;
							break;

						case Hexagon.COORDNUM_BOTTOMLEFT:
							curHex= getHexagon(x, y-1);
							entered= Hexagon.COORDNUM_TOPRIGHT;
							break;

						case Hexagon.COORDNUM_TOPLEFT:
							curHex= getHexagon(x-1, y-1);
							entered= Hexagon.COORDNUM_BOTTOMRIGHT;
							break;

						default:
							return hexList;
					}

					if (curHex != null)
					{
						hexList.add(curHex);
						found= true;
						break;
					}
				}
			}

			if (!found)
				return hexList;
			else found= false;

		}
		while (!curHex.isOccupied());

      return hexList;
	}

	/**
	 *  Returns the line-of-sight from hexagon at hex location (hexx1,hexy1).
	 *  This is done by drawing a line between the middle pixel of the source
	 *  hexagon to the middle pixel of the hexagon at (hexx2,hexy2) and seeing
	 *  which hexagons intersect with
	 *  the given line. A list of the intersecting hexagons is returned. If the
	 *  line of sight is blocked by an object on the way, the line of sight
	 *  is interrupted and does not continue. Note that it is assumed that objects
	 *  take up the full hexagon on which they reside and that they block the
	 *  sight of anything behind them.
	 */
   public List getLOS(int hexx1, int hexy1, int hexx2, int hexy2)
   {
		Hexagon dest= getHexagon(hexx2, hexy2);

		return getLOSPixel(hexx1, hexy1, dest.getMidPixelX(), dest.getMidPixelY());
	}

	/**
	 *  Get all the hexagons surround the given source hexagon (h) 
	 *  within the specified radius. The radius is the distance measured in hexagons
	 *  from the surrounding hexagons to the center hexagon. Only valid hexagons are
	 *  returned.
	 */
	public Hexagon[] getSurroundingArray(Hexagon h, int radius)
	{
		List list= getSurrounding(h.getHexX(), h.getHexY(), radius);

		Hexagon[] array= new Hexagon[list.size()];

		for (int i= 0; i < array.length; i++)
			array[i]= (Hexagon)list.get(i);

		return array;
	}
	
	/**
	 *  Get all the hexagons surround the given source hexagon at (hexx,hexy)
	 *  within the specified radius. The radius is the distance measured in hexagons
	 *  from the surrounding hexagons to the center hexagon. Only valid hexagons are
	 *  returned.
	 */
   public List getSurrounding(int hexx, int hexy, int radius)
   {
      List hexes= new LinkedList();

      if (radius < 0)
         return hexes;

      hexes.add(getHexagon(hexx, hexy));
      getSurrounding_rec(hexx, hexy, radius, hexes);

      return hexes;
   }

	// recursively add the list of surrounding hexes by increasing radius sizes
   private void getSurrounding_rec(int hexx, int hexy, int radius, List curList)
   {
      if (radius == 0)
      {
         return;
      }
      else
      {
         LinkedList oldList= new LinkedList(curList);

         for (int i= 0; i < oldList.size(); i++)
         {
            Hexagon h= (Hexagon)oldList.get(i);

            int cur_hexx= h.getHexX();
            int cur_hexy= h.getHexY();

            // left 1
            if (isValidCoord(cur_hexx, cur_hexy-1))
               if (!oldList.contains(getHexagon(cur_hexx, cur_hexy-1)))
                  curList.add(getHexagon(cur_hexx, cur_hexy-1));

            // right 1
            if (isValidCoord(cur_hexx, cur_hexy+1))
               if (!oldList.contains(getHexagon(cur_hexx, cur_hexy+1)))
                  curList.add(getHexagon(cur_hexx, cur_hexy+1));

            // northeast 1
            if (isValidCoord(cur_hexx-1, cur_hexy))
               if (!oldList.contains(getHexagon(cur_hexx-1, cur_hexy)))
                  curList.add(getHexagon(cur_hexx-1, cur_hexy));

            // northwest 1
            if (isValidCoord(cur_hexx-1, cur_hexy-1))
               if (!oldList.contains(getHexagon(cur_hexx-1, cur_hexy-1)))
                  curList.add(getHexagon(cur_hexx-1, cur_hexy-1));

            // southwest 1
            if (isValidCoord(cur_hexx+1, cur_hexy))
               if (!oldList.contains(getHexagon(cur_hexx+1, cur_hexy)))
                  curList.add(getHexagon(cur_hexx+1, cur_hexy));

            // southeast 1
            if (isValidCoord(cur_hexx+1, cur_hexy+1))
               if (!oldList.contains(getHexagon(cur_hexx+1, cur_hexy+1)))
                  curList.add(getHexagon(cur_hexx+1, cur_hexy+1));
         }

         getSurrounding_rec(hexx, hexy, radius-1, curList);
      }
   }

	/**
	 *  Get the list of hexagons in the Field-of-View from the source
	 *  hexagon at hex location (hexx1, hexy1). The field-of-view is
	 *  what can be seen from a 360-degree cone of sight centered at
	 *  the source, with any objects considered to be fully occupying
	 *  their hexagons (blocking the view behind them).
	 *
	 *  <b>Note</b>: the maxradius
	 *  parameter is not yet being used.
	 */
   public java.util.List getFOV(int hexx1, int hexy1, int maxradius)
	{
		List hexList= new LinkedList();
		List exterior;

		exterior= getExterior();

		Iterator i= exterior.listIterator();

		while (i.hasNext())
		{
			Hexagon h= (Hexagon)i.next();
			hexList= Misc.union(hexList,
				getLOS(hexx1, hexy1, h.getHexX(), h.getHexY()));
		}

		return hexList;
	}

	// ------------------------------------

	/**
	 *  An iterator over all tiles in the board.
	 */
   class TileIterator implements Iterator
   {
      private int x, y;
      HexagonalBoard b;

		/** Create a new iterator on the specified board. */
      public TileIterator(HexagonalBoard new_b)
      {
         b= new_b;
         x= y= 0;

         // go to the first valid coord
         for (int i= 0; i < sideLength; i++)
            for (int j= 0; j < sideLength; j++)
               if (b.isValidCoord(i,j))
               {
                  x= i;
                  y= j;
                  return;
               }
      }

		/**
		 *  Returns true if there is another hexagon in the iterator.
		 *  False otherwise.
		 */
      public boolean hasNext()
      {
         if (b.isValidCoord(x,y))
            return true;
         else
            return false;

      }

		/** Returns the next hexagon in the iterator. */
      public Object next()
      {
         int old_x= x, old_y= y;
         boolean done= false;
         boolean firstiter= true;
       
         if (!hasNext())
            return null;

         int i= 0, j= 0;

         for (i= x ; i < sideLength; i++)
         {
            for (j= (firstiter ? (y+1) : 0); j < sideLength; j++)
            {
               if (b.isValidCoord(i,j))
               {
                  done= true;
                  break;
               }
            }

            firstiter= false;

            if (done)
               break;

         }

         x= i;
         y= j;

         return tiles[old_x][old_y];
      }

		/** This method does nothing. */
      public void remove()
      {
      }
   }

}
